import 'dart:async';

import 'package:osam/domain/state/base_state.dart';
import 'package:osam/presentation/presenter.dart';

import '../osam.dart';

mixin NavigationPresenter<US extends BaseState<US>> on Presenter<US> {
  NavigationState get navigationState;

  StreamSubscription<List<String>> routesSub;

  @override
  void init() {
    routesSub = navigationState.routesStream.listen((routes) {});
  }

  StateStream<List<String>> get routesStream => navigationState.routesStream;

  void pop() => navigationState.pop();

  void push(String route) => navigationState.push(route);

  void pushAndRemoveUntil(String route) => navigationState.pushAndRemoveUntil(route);

  void pushReplacement(String route) => navigationState.pushReplacement(route);

  @override
  void dispose() {
    routesSub.cancel();
    super.dispose();
  }
}

mixin TabNavigationPresenter<US extends BaseState<US>> on Presenter<US> {
  TabNavigationState get tabNavigationState;

  void changeTab(String tab) => tabNavigationState.changeTab(tab);

  StateStream<String> get tabStream => tabNavigationState.tabStream;
}
